
clear;
run('./vlfeat-0.9.19/toolbox/vl_setup');
addpath('./others/');
%%------------------------set parameters---------------------%%
theta=10; % control the edge weight
imgRoot='./src/';% the input path
saldir='./test/';% the output path
mkdir(saldir);
imnames=dir([imgRoot '*' 'jpg']);
%matlabpool;

parfor ii=1:length(imnames)   
    imname=[imgRoot imnames(ii).name];
    input_im=imread(imname);
    dposition=strfind(imnames(ii).name,'.');
    position=dposition(1)-1;
    disp(imname);
    [ hsum, wsum, input_im] = ImageCrop(input_im);
    input_im=im2double(input_im);
    [m,n,k] = size(input_im);

%%----------------------generate superpixels--------------------%%

    superpixels = vl_slic(single(input_im), 20, 0.1)+1;
    spnum=double(max(superpixels(:)));
    miss=setdiff(1:spnum,unique(superpixels));
    for i=length(miss):-1:1
        superpixels(superpixels>miss(i))=superpixels(superpixels>miss(i))-1;
    end
    spnum=spnum-length(miss);% the actual superpixel number  

%%----------------------design the graph model--------------------------%%
% compute the feature (mean color in lab color space) 
% for each node (superpixels)
    input_vals=reshape(input_im, m*n, k);
    rgb_vals=zeros(spnum,1,3);
    inds=cell(spnum,1);
    Mcenter=zeros(spnum,2);
    [X, Y]=meshgrid(1:n,1:m);
    for i=1:spnum
        inds{i}=find(superpixels==i);
        rgb_vals(i,1,:)=mean(input_vals(inds{i},:),1);
        Mcenter(i,1)=mean(X(superpixels==i));
        Mcenter(i,2)=mean(Y(superpixels==i));
    end  
    lab_vals = colorspace('Lab<-', rgb_vals); 
    seg_vals=reshape(lab_vals,spnum,3);% feature for each superpixel
 
    
    % compute affinity matrix
    adjloop=AdjcProcloop2(superpixels,spnum,Mcenter,20);
    adjloop(adjloop>2)=0;              
       
    links=find(adjloop>0);
    edges=zeros(length(links),3);
    [edges(:,1),edges(:,2)]=ind2sub(size(adjloop),links);
    edges(:,3)=adjloop(links);
    edges=edges((edges(:,2)-edges(:,1))>0,:);

    weights = makeweights2(edges,seg_vals,theta);
    W = adjacency(double(edges),weights,spnum)+sparse(eye(spnum));

    %eigenvectors(Ur) and eigenvalues(LL) of Lrw
    dd = sum(W); D = sparse(1:spnum,1:spnum,dd);
    invD = (D\eye(spnum));
    Lsym=(invD^0.5)*(D-W)*(invD^0.5);  
    [Us, LL] = eigenshuffle(Lsym);
    Ur = (invD^0.5)*Us;
 
    %eigengap
    difL=[LL(2:end);0]-LL;
    difL=difL(1:end-1);
    [~,IX]=sort(difL,'descend');
    egap=IX(1);
    if egap==1
        egap=IX(2);
    end    

    %variance of each eigenvector after normalizing to [0 255]
    minUr=ones(spnum,1)*min(Ur);
    maxUr=ones(spnum,1)*max(Ur);
    FU=uint8(((Ur-minUr)./(maxUr-minUr))*255);FU(:,1)=255;
    VA=var(double(FU),0,1);
     
    %discard the first constant eigenvector
    invLL=LL.^-1;
    invLLegap=invLL(2:egap);
    VAegap=VA(2:egap);VAegap=VAegap>300;
    if egap==2&&VAegap==0
        Ur(:,egap)=Ur(:,egap+1);VAegap=1;
    end
    
    %re-synthesize diffusion matrix
    invA=Ur(:,2:egap)*diag(invLLegap)*diag(VAegap)*Ur(:,2:egap)';
 
    bsa=unique([superpixels(1,:),superpixels(m,:),superpixels(:,1)',superpixels(:,n)']);    
    tran=ones(spnum,1);tran(bsa)=0;
    
    sv = invA*tran;%seed vector
    fsal = invA*sv;%final saliency
    
    
    fsal=mat2gray(fsal);  
    tmapstage1=ToImage(fsal,inds,[m n],[hsum wsum],1);
    tmapstage2=imadjust(tmapstage1,stretchlim(tmapstage1,0.03),[]); %imadjust to increase the contrast of the final saliency maps.
    outname=[saldir imnames(ii).name(1:position) '_GP' '.png'];
    imwrite(tmapstage2,outname); 
    
end

%matlabpool close;


